package com.omega365.app_center_diagnostics_and_analytics

import android.app.Application
import androidx.annotation.NonNull

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import com.microsoft.appcenter.AppCenter
import com.microsoft.appcenter.analytics.Analytics
import com.microsoft.appcenter.crashes.Crashes
import com.microsoft.appcenter.crashes.ingestion.models.ErrorAttachmentLog

class AppCenterDiagnosticsAndAnalyticsPlugin: FlutterPlugin, MethodCallHandler {
  private lateinit var channel : MethodChannel
  private lateinit var application: Application

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    application = flutterPluginBinding.applicationContext as Application

    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "app_center_diagnostics_and_analytics")
    channel.setMethodCallHandler(this)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    try {
      when (call.method) {
        "start" -> {
          val appSecret = call.argument<String>("secret")

          AppCenter.start(application, appSecret, Analytics::class.java, Crashes::class.java)
        }
        "getInstallID" -> {
          result.success(AppCenter.getInstallId().get()?.toString())
          return
        }
        "configureAnalytics" -> {
          val value = call.arguments as Boolean
          Analytics.setEnabled(value).get()
        }
        "isAnalyticsEnabled" -> {
          result.success(Analytics.isEnabled().get())
          return
        }
        "trackEvent" -> {
          val name = call.argument<String>("name")
          val properties = call.argument<Map<String, String>>("properties")

          Analytics.trackEvent(name, properties)
        }
        "configureCrashes" -> {
          val value = call.arguments as Boolean
          Crashes.setEnabled(value).get()
        }
        "isCrashesEnabled" -> {
          result.success(Crashes.isEnabled().get())
          return
        }
        "trackCrash" -> {
          val throwable = call.argument<Throwable>("throwable")
          val properties = call.argument<Map<String, String>>("properties")

          val error = call.argument<String>("error")
          val exception = call.argument<String>("exception")
          val stackTrace = call.argument<String>("stackTrace")

          val attachments = mutableListOf<ErrorAttachmentLog>()

          if (error != null && error.isNotBlank()) {
            attachments.add(ErrorAttachmentLog.attachmentWithText(error, "error.txt"))
          }

          if (error != null && error.isNotBlank()) {
            attachments.add(ErrorAttachmentLog.attachmentWithText(exception, "exception.txt"))
          }

          if (error != null && error.isNotBlank()) {
            attachments.add(ErrorAttachmentLog.attachmentWithText(stackTrace, "stackTrace.txt"))
          }

          Crashes.trackError(throwable, properties, attachments)
        }
        else -> {
          result.notImplemented()
        }
      }
    } catch (error: Exception) {
      throw error;
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }
}
