import Flutter
import UIKit
import AppCenter
import AppCenterAnalytics
import AppCenterCrashes

public class SwiftAppCenterDiagnosticsAndAnalyticsPlugin: NSObject, FlutterPlugin {
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "app_center_diagnostics_and_analytics", binaryMessenger: registrar.messenger())
    let instance = SwiftAppCenterDiagnosticsAndAnalyticsPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method {
      case "start":
        start(call: call, result: result)
      case "getInstallID":
        result(AppCenter.installId().uuidString)
        return
      case "configureAnalytics":
        Analytics.setEnabled(call.arguments as! Bool)
      case "isAnalyticsEnabled":
        result(Analytics.isEnabled())
        return
      case "trackEvent":
        Analytics.trackEvent(call: call, result: result)
        return
      case "configureCrashes":
        Crashes.setEnabled(call.arguments as! Bool)
      case "isCrashesEnabled":
        result(Crashes.isEnabled())
        return
      case "trackCrash":
        trackCrash(call: call, result: result)
        return
      default:
        result(FlutterMethodNotImplemented)
        return
    }

    result(nil);
  }

  private func start(call: FlutterMethodCall, result: FlutterResult) {
    guard let args:[String: Any] = (call.arguments as? [String: Any]) else {
      result(FlutterError(code: "400", message:  "Bad arguments", details: "iOS could not recognize flutter arguments in method: (start)"))
      return
    }

    let secret = args["secret"] as! String

    AppCenter.start(withAppSecret: secret, services:[Analytics.self, Crashes.self])
  }

  private func trackEvent(call: FlutterMethodCall, result: FlutterResult) {
    guard let args:[String: Any] = (call.arguments as? [String: Any]) else {
      result(FlutterError(code: "400", message:  "Bad arguments", details: "iOS could not recognize flutter arguments in method: (trackEvent)") )
      return
    }

    let name = args["name"] as! String
    let properties = args["properties"] as? [String: String]

    Analytics.trackEvent(name!, withProperties: properties)
  }
}
